#####################
####  R codes of the 4th lecture of the applied spatial econometrics course
####  Spatial Econometrics Models with R
####  T. Laurent (15/03/2016) 

###################################
#####  Slide p.3
###################################

source("lecture3.r")

form <- log(MEDV) ~ CRIM + ZN + INDUS + CHAS + I(NOX^2) + 
 I(RM^2) +  AGE + log(DIS) + log(RAD) + TAX + PTRATIO + 
 B + log(LSTAT)
 
boston.lm <- lm(form, data=boston.tr@data)

res.boston <- residuals(boston.lm)


###################################################
### To get the OLS summary table in p.4 
###################################################
## library(stargazer) 
## stargazer(boston.lm)


###################################
#####  Slide p.5
###################################

boston.ex_knn <- knearneigh(coord, k=5, longlat = FALSE)
boston.ex_nb <- knn2nb(boston.ex_knn)   
boston_listw2 <- nb2listw(boston.ex_nb)

###################################
#####  Slide p.6
###################################

morantest1 <- lm.morantest(boston.lm, boston_listw)
morantest2 <- lm.morantest(boston.lm, boston_listw2)


###################################################
### To get the results as presented in the Table
###################################################
require("xtable")
res1 <- matrix("", ncol=5, nrow=2)
rownames(res1) <- c("W1", "W2")
colnames(res1) <- c("$I$", "$E(I)$", "$var(I)$", "st. deviate", "$p$-value")
res1[1, 1:3] <- format(morantest1$estimate, digits=2)
res1[1, 4] <- format(morantest1$statistic, digits=3)
res1[1, 5] <- format.pval(morantest1$p.value, digits=2, eps=1e-8)
res1[2, 1:3] <- format(morantest2$estimate, digits=2)
res1[2, 4] <- format(morantest2$statistic, digits=3)
res1[2, 5] <- format.pval(morantest2$p.value, digits=2, eps=1e-8)
print(xtable(res1, align=c("l", rep("r", 5))), floating=FALSE,
 sanitize.text.function = function(SANITIZE) SANITIZE)


###################################
#####  Slide p.7
###################################

require("RColorBrewer")
require("GISTools")
plot.M(boston.tr, boston_listw, res.boston, rev(brewer.pal(4,"Spectral")), "residuals")


###################################
#####  Slide p.12
###################################

LM.boston <- lm.LMtests(boston.lm, listw=boston_listw, test="all")
LM.boston2 <- lm.LMtests(boston.lm, listw=boston_listw2, test="all")


###################################################
### To get the results as presented in the Table
###################################################
tres <- t(sapply(LM.boston, function(x) c(x$statistic, x$parameter, x$p.value)))
colnames(tres) <- c("Stat. (W1)", "df", "p-value")
tres2 <- t(sapply(LM.boston2, function(x) c(x$statistic, x$parameter, x$p.value)))
colnames(tres2) <- c("Stat. (W2)", "df", "p-value")
xtable(cbind(tres[,c("df","Stat. (W1)","p-value")], tres2[,c("Stat. (W2)", "p-value")]))


###################################
#####  Slide p.13
###################################

lag.boston <- lagsarlm(form, data=boston.tr, boston_listw)
sdm.boston <- lagsarlm(form, data=boston.tr, boston_listw, 
 type="mixed")
sac.boston <- sacsarlm(form, data=boston.tr, boston_listw)
sem.boston <- errorsarlm(form, data=boston.tr, boston_listw)

lag.boston2 <- lagsarlm(form, data=boston.tr, boston_listw2)
sdm.boston2 <- lagsarlm(form, data=boston.tr, boston_listw2, 
 type="mixed")
sac.boston2 <- sacsarlm(form, data=boston.tr, boston_listw2)
sem.boston2 <- errorsarlm(form, data=boston.tr, boston_listw2)


###################################################
### To get the results as presented in the Table
###################################################
res1<-AIC(lag.boston, sdm.boston, sac.boston, sem.boston)
res2<-AIC(lag.boston2, sdm.boston2, sac.boston2, sem.boston2)
res <- data.frame(df=res1$df, AIC.W1=res1$AIC, AIC.W2=res2$AIC)
row.names(res) <- c("SAR","SDM","SAC","SEM")
xtable(res)


###################################
#####  Slide p.14
###################################
res <- data.frame(parameters=c(as.vector(t(cbind(names(coefficients(sdm.boston)[1:10]),""))),"...", "AIC","sigma.square"), 
W1=c(as.vector(t(cbind(round(c(coefficients(sdm.boston)[1:10]),6), paste("(",round(sqrt(diag(sdm.boston$resvar)[2:10]),6),")",sep="")))),"...",round(AIC(sdm.boston),2),round(sdm.boston$s2,4)),  
W2=c(as.vector(t(cbind(round(c(coefficients(sdm.boston2)[1:10]),6), paste("(",round(sqrt(diag(sdm.boston2$resvar)[2:10]),6),")",sep="")))),"...", round(AIC(sdm.boston2),2), round(sdm.boston2$s2,4))
) 
print(xtable(res),include.rownames=F)


###################################################
### Initialize the following values for estimating the model 
###################################################
y <- boston.lm$model[,1] 
lag.y <- lag(boston_listw, y) 
n <- length(y)
x2 <- boston.lm$model[,-1]
x2$CHAS <- as.numeric(as.character(x2$CHAS))
x <- as.matrix(cbind(rep(1,n), x2)) 
colnames(x)[1] <- "Intercep"


###################################
#####  Slide p.17
###################################
Lc <- function(rho) Jacobian(rho)-n/2*log(S(rho)) 

W <- listw2mat(boston_listw)
eigen.values <- eigen(W)
bound.lambda <- 1/range(eigenw(boston_listw))

A <- function(rho) diag(n)-rho*W
Jacobian <- function(rho) log(prod(1-rho*eigen.values$values))


###################################
#####  Slide p.18
###################################

bo <- solve(qr(crossprod(x)), crossprod(x, y))
eo <- y - x%*%bo 
bd <- solve(qr(crossprod(x)), crossprod(x, lag.y))
ed <- lag.y - x%*%bd 
S <- function(rho) crossprod(eo) - 2*rho*crossprod(eo,ed) + 
 rho^2*crossprod(ed)


###################################
#####  Slide p.19
###################################
(hat.rho <- optimize(Lc, bound.lambda, maximum = T)$maximum) 

x.eval <- seq(bound.lambda[1], bound.lambda[2], by=0.01)
plot(x.eval, Lc(x.eval), type="l", xlab="rho", ylab="Lc")


###################################
#####  Slide p.20
###################################
hat.beta <- bo - hat.rho*bd
hat.sigma <- as.numeric(1/n*S(hat.rho))


###################################
#####  Slide p.21
###################################
resvar <- lag.boston$resvar

W.sparse <-  as(as_dgRMatrix_listw(boston_listw), 
 "CsparseMatrix") 


###################################
#####  Slide p.22
###################################
etr_mult <- trW(W.sparse, m = 24, type = "mult")
system.time(lag.boston2 <- lagsarlm(form, data=boston.tr, 
 boston_listw, method="Matrix", tr = etr_mult))

etr_MC <- trW(W.sparse, m = 24, type = "MC")
system.time(lag.boston3 <- lagsarlm(form, data=boston.tr, 
 boston_listw, method="MC", tr = etr_MC))


###################################
#####  Slide p.23
###################################
res <- data.frame(parameters=c(as.vector(t(cbind(names(coefficients(lag.boston)[1:10]),""))),"...", "AIC","sigma.square"), 
eigen=c(as.vector(t(cbind(round(c(coefficients(lag.boston)[1:10]),4), paste("(",round(sqrt(diag(lag.boston$resvar)[2:10]),4),")",sep="")))),"...",round(AIC(lag.boston),2),round(lag.boston$s2,4)),  
Matrix.mult=c(as.vector(t(cbind(round(c(coefficients(lag.boston2)[1:10]),4), paste("(",round(sqrt(diag(lag.boston2$resvar)[2:10]),4),")",sep="")))),"...", round(AIC(lag.boston2),2), round(lag.boston$s2,4)),  
MC.mc=c(as.vector(t(cbind(round(c(coefficients(lag.boston3)[1:10]),4), paste("(",round(sqrt(diag(lag.boston3$resvar)[2:10]),4),")",sep="")))),"...", round(AIC(lag.boston3),2), round(lag.boston$s2,4)) ) 
print(xtable(res),include.rownames=F)


###################################
#####  Slide p.24
###################################

LR.test <- -2*(Lc(0)-Lc(hat.rho))
dchisq(LR.test, 1)

z.value <- (hat.rho - 0)/sqrt(resvar[2,2])
dnorm(z.value)

wald.test <- (hat.rho - 0)^2/resvar[2,2]
dchisq(wald.test)

lag.boston$LMtest


###################################
#####  Slide p.25
###################################
A.inv <- invIrW(boston_listw, hat.rho, method="solve") 
Y.TC <- A.inv%*%x%*%hat.beta

Y.TS <- x%*%hat.beta + hat.rho*W%*%y 

Q <- crossprod(A(hat.rho))/hat.sigma
diag.Q <- diag(n)*diag(Q)
diag.Q.inv <- diag(n)/diag(Q) 
Y.BP <- Y.TC - diag.Q.inv%*%(Q-diag.Q)%*%(y-Y.TC)


###################################
#####  Slide p.31
###################################
plotclr <- brewer.pal(5,"Blues")
A.inv.1 <- solve(diag(n)-0.5*listw2mat(boston_listw))
direct.ind.1 <- diag(A.inv.1)
A.inv.2 <- solve(diag(n)-0.5*listw2mat(boston_listw2))
direct.ind.2 <- diag(A.inv.2)
require("classInt")
 
##### We plot A^{-1} with W1
bk=round(classIntervals(direct.ind.1, 5,"quantile")$brks, 5)
ind=findInterval(direct.ind.1, bk, all.inside=TRUE)
op <- par(mfrow=c(1,2))
plot(boston.tr, col=plotclr[ind], main="\n \n \n \n \n \n \n \n W1")
decoup <- c(paste("<=",bk[2],sep=""),paste("]",bk[2],";",bk[3],"]", sep=""),
paste("]",bk[3],";",bk[4],"]", sep=""),paste("]",bk[4],";",bk[5],"]", sep=""),
paste(">",bk[5],sep=""))
legend("topleft", legend = decoup, cex = 1,title="Direct impacts", fill=plotclr)
 
##### We plot A^{-1} with W2
bk=round(classIntervals(direct.ind.2, 5,"quantile")$brks, 5)
ind=findInterval(direct.ind.2, bk, all.inside=TRUE)
plot(boston.tr, col=plotclr[ind], main="\n \n \n \n \n \n \n \n W2")
decoup <- c(paste("<=",bk[2],sep=""),paste("]",bk[2],";",bk[3],"]", sep=""),
paste("]",bk[3],";",bk[4],"]", sep=""),paste("]",bk[4],";",bk[5],"]", sep=""),
paste(">",bk[5],sep=""))
legend("topleft", legend = decoup, cex = 1,title="Direct impacts", fill=plotclr)
par(op)


###################################
#####  Slide p.32
###################################
plotclr <- brewer.pal(5,"Blues")
indirect.ind.1 <- apply(A.inv.1, 1, sum) - direct.ind.1
indirect.ind.2 <- apply(A.inv.2, 1, sum) - direct.ind.2
require("classInt")
 
##### We plot A^{-1} with W1
bk=round(classIntervals(indirect.ind.1, 5,"quantile")$brks, 5)
ind=findInterval(indirect.ind.1, bk, all.inside=TRUE)
op <- par(mfrow=c(1,2))
plot(boston.tr, col=plotclr[ind], main="\n \n \n \n \n \n \n \n W1")
decoup <- c(paste("<=",bk[2],sep=""),paste("]",bk[2],";",bk[3],"]", sep=""),
 paste("]",bk[3],";",bk[4],"]", sep=""),paste("]",bk[4],";",bk[5],"]", sep=""),
 paste(">",bk[5],sep=""))
legend("topleft", legend = decoup, cex = 1,title="Indirect impacts", fill=plotclr)
 
##### We plot A^{-1} with W2
bk=round(classIntervals(indirect.ind.2, 5,"quantile")$brks, 5)
ind=findInterval(indirect.ind.2, bk, all.inside=TRUE)
plot(boston.tr, col=plotclr[ind], main="\n \n \n \n \n \n \n \n W2")
decoup <- c(paste("<=",bk[2],sep=""),paste("]",bk[2],";",bk[3],"]", sep=""),
 paste("]",bk[3],";",bk[4],"]", sep=""),paste("]",bk[4],";",bk[5],"]", sep=""),
 paste(">",bk[5],sep=""))
legend("topleft", legend = decoup, cex = 1,title="Indirect impacts", fill=plotclr)
par(op)


###################################
#####  Slide p.33
###################################
direct <- sum(diag(A.inv))/n*coefficients(lag.boston)[-c(1,2)]

total <- sum(A.inv)/n*coefficients(lag.boston)[-c(1,2)]

indirect <- total - direct


###################################
#####  Slide p.34
###################################
impacts(lag.boston, listw=boston_listw)

impacts.sar <- impacts(lag.boston2, tr=etr_mult)
impacts(lag.boston3, tr=etr_MC)

###################################
#####  Slide p.35
###################################

lag.impacts <- impacts(lag.boston2, tr=etr_mult, R=200)

res.impacts <- summary(lag.impacts, zstats=T)

res.table <- data.frame(Direct=paste(round(res.impacts$res$direct,5), ifelse(res.impacts$pzmat[,1]<0.001," (***)", 
                                      ifelse(res.impacts$pzmat[,1]<0.01 & res.impacts$pzmat[,1]>0.001," (**)",
                                         ifelse(res.impacts$pzmat[,1]<0.05 & res.impacts$pzmat[,1]>0.01," (*)",""))),sep=""), 
                        Indirect=paste(round(res.impacts$res$indirect,5), ifelse(res.impacts$pzmat[,2]<0.001," (***)", 
                                      ifelse(res.impacts$pzmat[,2]<0.01 & res.impacts$pzmat[,2]>0.001," (**)",
                                         ifelse(res.impacts$pzmat[,2]<0.05 & res.impacts$pzmat[,2]>0.01," (*)",""))),sep=""),
                        Total=paste(round(res.impacts$res$total,5), ifelse(res.impacts$pzmat[,3]<0.001," (***)", 
                                      ifelse(res.impacts$pzmat[,3]<0.01 & res.impacts$pzmat[,3]>0.001," (**)",
                                         ifelse(res.impacts$pzmat[,3]<0.05 & res.impacts$pzmat[,3]>0.01," (*)",""))),sep="")) 
rownames(res.table) <- names(res.impacts$pzmat[,1])                                                                       
print(xtable(res.table))



###################################
#####  Slide p.36
###################################

lag.impacts <- impacts(lag.boston2, tr=etr_mult, R=200, Q=7)

res.impacts <- summary(lag.impacts, zstats=T, reportQ=TRUE)

res.table <- data.frame(direct=round(attr(res.impacts$res, "Qres")$direct,5)[,1],
indirect= round(attr(res.impacts$res, "Qres")$indirect,5)[,1],
total=round(attr(res.impacts$res, "Qres")$total,5)[,1])
rownames(res.table) <- paste("W",0:6, sep="")                                                                    
res.table = rbind(res.table, Total=apply(res.table,2,sum))
print(xtable(res.table, digits = 5))


