# required packages
require("maptools")
require("spdep")
# import the data
boston.tr <- readShapePoly(system.file("etc/shapes/boston_tracts.shp",
  package="spdep")[1], ID="poltract",
  proj4string=CRS(paste("+proj=longlat +datum=NAD27 +no_defs +ellps=clrk66",
  "+nadgrids=@conus,@alaska,@ntv2_0.gsb,@ntv1_can.dat")))

  # change the CRS
boston.tr <- spTransform(boston.tr,CRS("+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +no_defs"))
   
# the ID's of the tracts
ID <- row.names(boston.tr)
n.boston <- length(ID)

# we define the centroid of the tracts
coord <- coordinates(boston.tr)

# Spatial neighbours
boston_nb.1 <- poly2nb(boston.tr)
boston_nb.2 <- dnearneigh(coord, 0, 2000, longlat = FALSE, row.names=ID)
boston_nb <- union.nb(boston_nb.1, boston_nb.2)
# we build the spatial weights
dlist <- lapply(nbdists(boston_nb, coord), function(x) 1/x)
boston_listw <- nb2listw(boston_nb, glist=dlist, style="W")

# the sparse matrix
boston_sparse <- as_dgRMatrix_listw(boston_listw)

# moran scatter plot
plot.M <- function(sp.obj, listw.obj, x, col.quad, label)
{# sp.obj: a Spatial object
 # listw.obj: a spatial weights object
 # x: a numeric vector
 # col.quad: a vector of 4 colors for the quadrants
 # label: the name of the variable

 # lag variable
 wx <- lag.listw(listw.obj, x)

 # quadrants
 x1 <- which((x>mean(x))&(wx>mean(wx)))
 x2 <- which((x>mean(x))&(wx<mean(wx)))
 x3 <- which((x<mean(x))&(wx<mean(wx)))
 x4 <- which((x<mean(x))&(wx>mean(wx)))

 # quadrant colors
 q.col <- rep("white", length(x))
 q.col[x1] <- col.quad[1]
 q.col[x2] <- col.quad[3]
 q.col[x3] <- col.quad[2]
 q.col[x4] <- col.quad[4]

 # local moran index
 resI <- localmoran(x, listw.obj, alternative="two.sided")

 resI1 <- which((resI[,5]<0.05)&(x>mean(x))&(wx>mean(wx)))
 resI2 <- which((resI[,5]<0.05)&(x>mean(x))&(wx<mean(wx)))
 resI3 <- which((resI[,5]<0.05)&(x<mean(x))&(wx<mean(wx)))
 resI4 <- which((resI[,5]<0.05)&(x<mean(x))&(wx>mean(wx)))

 # plot
 layout(matrix(c(0,1,1,0,0,1,1,0,2,2,3,3,2,2,3,3), 4, 4, byrow = TRUE))
 # 1st graph
 res.moran<-moran.plot(x, listw.obj, main=paste("Moran plot of", label),
  xlab=label, ylab=paste("W x", label), col=q.col, pch=16, cex=0.5, labels=F)
 points(x[c(resI1, resI2, resI3, resI4)], wx[c(resI1, resI2, resI3, resI4)],
  col=q.col[c(resI1, resI2, resI3, resI4)], pch=16, cex=1.25)
 legend("bottomright",c("H-H","H-L","L-L","L-H"),
  fill=col.quad[c(1,3,2,4)],title="Cluster",cex=0.8)
 legend("topleft", c("Influent point", "LISA sign.", "LISA non sign."),
  pch=c(9,16,16), pt.cex=c(1,1.4,0.5))

 # 2nd graph
 plot(sp.obj, col=add.alpha(q.col,0.3), border=add.alpha(q.col,0.3))
 which.inf <- which(apply(res.moran$is.inf,1,any))
 if(length(which.inf)>0)
  plot(sp.obj[which.inf,],col=q.col[which.inf], add=T, lwd=1)
 title("Influent points cluster map")
 legend("topleft",c("H-H","H-L","L-L","L-H"),fill=col.quad[c(1,3,2,4)],title="Cluster",cex=0.8)

 # 3rd graph
 plot(sp.obj, col=add.alpha(q.col,0.3), border=add.alpha(q.col,0.3))
 which.lisa <- c(resI1, resI2, resI3, resI4)
 if(length(which.lisa)>0)
 plot(sp.obj[which.lisa,],   col=q.col[which.lisa], add=T, lwd=1)
 title("Significant LISA cluster map")
 legend("topleft",c("H-H","H-L","L-L","L-H"),fill=col.quad[c(1,3,2,4)],title="Cluster",cex=0.8)
}

# applications :
# plot.M(boston.tr, boston_listw, res.boston, rev(brewer.pal(4,"Spectral")), "residuals")
